/**********************************************************
						WARPOFDMTX.H
WARPOFDM is the driver that allows user code to manipulate
states of both the OFDM transmitter and receiver.
Additionally, it allows direct control of certain behaviour
of PHY.

Author: Chris Hunter 5/25/06
***********************************************************/

#ifndef WARPOFDMTX_H
#define WARPOFDMTX_H


//********Prototypes********

void mimo_ofdmTx_init();
void mimo_ofdmTx_enable();
void mimo_mac2phy_pktTx(unsigned int block, unsigned int radio);
void mimo_ofdmTx_setControlBits(unsigned int txOptions);

//********Defines********
//Tx Control Options
#define TX_SISO_MODE		0x1
#define TX_RANDOM_MODE		0x2
#define TX_DISABLE_ANTB_PREAMBLE 0x4
#define TX_PILOT_SCRAMBLING 0x8


//MAC2PHY Options
#define TXBLOCK			0x0
#define TXNOBLOCK			0x1


//Macros



#define mimo_ofdmTx_setFFTScaling(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_FFT_Scaling_OFFSET))= c
#define mimo_ofdmTx_setPilot1Index(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_Pilots_Index1_OFFSET)) = c
#define mimo_ofdmTx_setPilot1Value(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_Pilots_Value1_OFFSET)) = c
#define mimo_ofdmTx_setPilot2Index(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_Pilots_Index2_OFFSET)) = c
#define mimo_ofdmTx_setPilot2Value(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_Pilots_Value2_OFFSET)) = c
#define mimo_ofdmTx_setRandom(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_RandomPayload_ModulationSelt_OFFSET)) = c
#define mimo_ofdmTx_setNumBytes(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_NumPayloadBytes_OFFSET)) = c
#define mimo_ofdmTx_setNumSyms(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_OFDM_SymCounts_OFFSET)) = c

#define mimo_ofdmTx_setPktDoneReset(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_Start_Reset_Control_OFFSET)) = (c<<2)&0x4



#define mimo_ofdmTx_setPreambleScaling(c) *((volatile unsigned int*)(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_Tx_PreambleScaling_OFFSET)) = c


#define mimo_ofdmTx_copyHeader(src,len) memcpy(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_SMWO_TxPktBuffer_OFFSET,(src),(len))
#define mimo_ofdmTx_copyPayload(src,len) memcpy(XPAR_OFDM_TXRX_MIMO_OPBW_0_BASEADDR+ofdm_TxRx_mimo_SMWO_TxPktBuffer_OFFSET+6*sizeof(int),(src),(len))


#endif
